@@ -1,3 +1,4 @@
+#define PERL_NO_GET_CONTEXT
 #include "EXTERN.h"
 #include "perl.h"
 #include "XSUB.h"
@@ -6,7 +7,10 @@
 
 #include <SWI-Prolog.h>
 
+#include "context.h"
+
 #include "frames.h"
+#include "engines.h"
 #include "perl2swi.h"
 #include "swi2perl.h"
 #include "query.h"
@@ -34,13 +38,17 @@ static PL_extension callperl[] = {
     { NULL, 0, NULL, 0 }, };
 
 void boot_callperl(void) {
-    PL_register_extensions(callperl);
+    static int registered=0;
+    if(!registered) {
+	registered=1;
+	PL_register_extensions(callperl);
+    }
 }
 
-static savestate(void) {
-    savestate_Low();
-    savestate_query();
-    savestate_vars();
+static savestate(pTHX_ pMY_CXT) {
+    savestate_Low(aTHX_ aMY_CXT);
+    savestate_query(aTHX_ aMY_CXT);
+    savestate_vars(aTHX_ aMY_CXT);
     /*
       save_hptr(&PL_curstash);
       save_item(PL_curstname);
@@ -49,9 +57,9 @@ static savestate(void) {
     */
 }
 
-static int test_oq(void) {
-    if (isquery()) {
-	cutquery();
+static int test_oq(pTHX_ pMY_CXT) {
+    if (is_query(aTHX_ aMY_CXT)) {
+	cut_query(aTHX_ aMY_CXT);
 	return 1;
     }
     return 0;
@@ -70,13 +78,14 @@ static void oq_exception(term_t *e) {
 		  PL_CHARS, "open_query");
 }
 
-static int pop_results(int nret, term_t *r, term_t *e) {
+static int pop_results(pTHX_ int nret, term_t *r, term_t *e) {
     AV *vars=(AV*)sv_2mortal((SV *)newAV());
     AV *cells=(AV*)sv_2mortal((SV *)newAV());
     if (!nret && SvTRUE(ERRSV)) {
 	PL_unify_term(*e=PL_new_term_ref(),
 		      PL_FUNCTOR_CHARS, "perl_exception", 1,
-		      PL_TERM, perl2swi_sv(ERRSV,
+		      PL_TERM, perl2swi_sv(aTHX_
+					   ERRSV,
 					   vars, cells));
 	return 0;
     }
@@ -86,7 +95,7 @@ static int pop_results(int nret, term_t *r, term_t *e) {
 	PL_put_nil(ret);
 	while(--nret>=0) {
 	    PL_cons_list(ret,
-			 perl2swi_sv(POPs, vars, cells),
+			 perl2swi_sv(aTHX_ POPs, vars, cells),
 			 ret);
 	}
 	*r=ret;
@@ -94,18 +103,18 @@ static int pop_results(int nret, term_t *r, term_t *e) {
     }
 }
 
-static int push_args(term_t obj, int obj_ok, term_t args, term_t *e) {
+static int push_args(pTHX_ term_t obj, int obj_ok, term_t args, term_t *e) {
     dSP;
     term_t head, list;
     AV *cells=(AV *)sv_2mortal((SV *) newAV());
     if (obj_ok) {
-	XPUSHs(sv_2mortal(swi2perl(obj,cells)));
+	XPUSHs(sv_2mortal(swi2perl(aTHX_ obj,cells)));
     }
     head=PL_new_term_ref();
     list=PL_copy_term_ref(args);
     while (!PL_get_nil(list)) {
 	if (PL_get_list(list, head, list)) {
-	    XPUSHs(sv_2mortal(swi2perl(head, cells)));
+	    XPUSHs(sv_2mortal(swi2perl(aTHX_ head, cells)));
 	}
 	else {
 	    PL_unify_term(*e=PL_new_term_ref(),
@@ -125,22 +134,24 @@ static foreign_t swi2perl_sub(term_t name,
     char *cname;
     term_t e;
     if (PL_get_atom_chars(name, &cname)) {
+	MY_dTHX;
+	dMY_CXT;
 	dSP;
 	term_t ret;
 	int nret, oq, ok;
 
 	ENTER;
 	SAVETMPS;
-	savestate();
+	savestate(aTHX_ aMY_CXT);
 
 	PUSHMARK(SP);
-	if (ok=push_args(0, 0, args, &e)) {
-	    push_frame();
+	if (ok=push_args(aTHX_ 0, 0, args, &e)) {
+	    push_frame(aTHX_ aMY_CXT);
 	    nret=call_pv(cname, G_ARRAY | G_EVAL);
 	    SPAGAIN;
-	    oq=test_oq();
-	    pop_frame();
-	    if (ok=pop_results(nret, &ret, &e)) {
+	    oq=test_oq(aTHX_ aMY_CXT);
+	    pop_frame(aTHX_ aMY_CXT);
+	    if (ok=pop_results(aTHX_ nret, &ret, &e)) {
 		if (oq) {
 		    ok=0;
 		    oq_exception(&e);
@@ -163,23 +174,28 @@ static foreign_t swi2perl_eval(term_t code,
     char *ccode;
     term_t e;
     if (PL_get_atom_chars(code, &ccode)) {
+	MY_dTHX;
+	dMY_CXT;
 	dSP;
 	term_t ret;
 	int nret, oq, ok;
 	SV *svcode;
-
+	fprintf(stderr, "pre eval\n");
+	fflush(stderr);
 	ENTER;
 	SAVETMPS;
-	savestate();
+	savestate(aTHX_ aMY_CXT);
 	PUSHMARK(SP);
 
-	push_frame();
+	push_frame(aTHX_ aMY_CXT);
 	svcode=sv_2mortal(newSVpv(ccode, 0));
+	fprintf(stderr, "entering eval\n");
 	nret=eval_sv(svcode, G_ARRAY | G_EVAL);
 	SPAGAIN;
-	oq=test_oq();
-	pop_frame();
-	if (ok=pop_results(nret, &ret, &e)) {
+	fprintf(stderr, "returned from eval\n");
+	oq=test_oq(aTHX_ aMY_CXT);
+	pop_frame(aTHX_ aMY_CXT);
+	if (ok=pop_results(aTHX_ nret, &ret, &e)) {
 	    if (oq) {
 		ok=0;
 		oq_exception(&e);
@@ -202,22 +218,24 @@ static foreign_t swi2perl_method(term_t object,
     char *cmethod;
     term_t e;
     if (PL_get_atom_chars(method, &cmethod)) {
+	MY_dTHX;
+	dMY_CXT;
 	dSP;
 	term_t ret;
 	int nret, oq, ok;
 
 	ENTER;
 	SAVETMPS;
-	savestate();
+	savestate(aTHX_ aMY_CXT);
 
 	PUSHMARK(SP);
-	if (ok=push_args(object, 1, args, &e)) {
-	    push_frame();
+	if (ok=push_args(aTHX_ object, 1, args, &e)) {
+	    push_frame(aTHX_ aMY_CXT);
 	    nret=call_method(cmethod, G_ARRAY|G_EVAL);
 	    SPAGAIN;
-	    oq=test_oq();
-	    pop_frame();
-	    if (ok=pop_results(nret, &ret, &e)) {
+	    oq=test_oq(aTHX_ aMY_CXT);
+	    pop_frame(aTHX_ aMY_CXT);
+	    if (ok=pop_results(aTHX_ nret, &ret, &e)) {
 		if (oq) {
 		    ok=0;
 		    oq_exception(&e);
