@@ -1,3 +1,4 @@
+#define PERL_NO_GET_CONTEXT
 #include "EXTERN.h"
 #include "perl.h"
 #include "XSUB.h"
@@ -6,38 +7,32 @@
 
 #include <SWI-Prolog.h>
 
-
+#include "context.h"
 #include "Low.h"
 #include "frames.h"
 
-AV *fids;
 
-void boot_frames(void) {
-    fids=get_av(PKG "::fids", 1);
-    SvREFCNT_inc(fids);
-}
-
-fid_t frame(void) {
+fid_t frame(pTHX_ pMY_CXT) {
     SV **w;
-    int len=av_len(fids);
+    int len=av_len(c_fids);
     if (len<0) {
 	die ("frame called and frame stack is empty");
     }
-    w=av_fetch(fids, len, 0);
+    w=av_fetch(c_fids, len, 0);
     if (!w) {
 	die ("corrupted frame stack");
     }
     return SvIV(*w);
 }
 
-void push_frame(void) {
+void push_frame(pTHX_ pMY_CXT) {
     SV *fid=newSViv(PL_open_foreign_frame());
     /* warn ("push_frame(%_)", fid); */
-    av_push(fids, fid);
+    av_push(c_fids, fid);
 }
 
-void pop_frame(void) {
-    SV *fid=av_pop(fids);
+void pop_frame(pTHX_ pMY_CXT) {
+    SV *fid=av_pop(c_fids);
     /* warn ("pop_frame(%_)", fid); */
     if (!SvOK(fid)) {
 	die ("pop_frame called but frame stack is empty");
@@ -46,8 +41,8 @@ void pop_frame(void) {
     SvREFCNT_dec(fid);
 }
 
-void rewind_frame(void) {
-    fid_t fid=frame();
+void rewind_frame(pTHX_ pMY_CXT) {
+    fid_t fid=frame(aTHX_ aMY_CXT);
     /* warn ("rewind_frame(%i)", fid); */
     PL_rewind_foreign_frame(fid);
 }
